const mongodb = require('mongodb');

const { ObjectId } = mongodb;

function findByName(name) {
  return { name };
}

function findById(id) {
  return { _id: ObjectId(id) };
}

function findByAttributes(objConstraints, searchFields) {
  let findQuery = {};
  /**
     * recordsNumber, pageNumber, includeTotalRows
     * ordering: [field, order]
     * simpleFilter: ['', '']
     * filtersByValue:[{ field, operator, value}]
     */
  if (objConstraints && objConstraints.simpleFilter) {
    const textToFind = objConstraints.simpleFilter[0];
    const queryRestrictions = [];
    searchFields.forEach((searchField) => {
      const fieldQuery = {};
      fieldQuery[searchField] = { $regex: textToFind };
      queryRestrictions.push(fieldQuery);
    });
    findQuery = { $or: queryRestrictions };
  } else if (objConstraints && objConstraints.filtersByValue) {
    objConstraints.filtersByValue.forEach((fieldFilter) => {
      const { field, operator, value } = fieldFilter;
      if (searchFields.includes(field)) {
        if (operator === '==') {
          findQuery[field] = value;
        }
      }
    });
  }
  return findQuery;
}

function updateObj(appData) {
  const querySet = { ...appData };
  if (querySet.additionalData) {
    Object.entries(querySet.additionalData).forEach(([key, val]) => {
      querySet[`additionalData.${key}`] = val;
    });
    delete querySet.additionalData;
  }
  return { $set: { ...querySet } };
}

function addExternalRef(objectData) {
  return { $push: { externalCoding: { ...objectData } } };
}

function findByExternalRef(externalSource, externalCode) {
  return { externalCoding: { externalSource, externalCode } };
}

function removeExternalRef(externalSource) {
  return { $pull: { externalCoding: { externalSource } } };
}

function findByUser(userId) {
  return { users: { userId: ObjectId(userId) } };
}

function addToArray(arrayName, data) {
  return { $addToSet: { [arrayName]: data } };
}

function removeFromArray(arrayName, identifier) {
  return { $pull: { [arrayName]: identifier } };
}

module.exports = {
  findByName,
  findById,
  updateObj,
  findByAttributes,
  addExternalRef,
  findByExternalRef,
  removeExternalRef,
  findByUser,
  addToArray,
  removeFromArray,
};
