const mongodb = require('mongodb');
const commonQueries = require('./common');

const STORE_GROUPS = 'StoreGroups';

const { ObjectId } = mongodb;

function findStores(thirdPartyId, storeIds) {
  const query = {};
  if (Array.isArray(storeIds)) {
    const thirdIds = storeIds.map(item => ObjectId(item));
    Object.assign(query, { _id: { $in: thirdIds } });
  } else if (thirdPartyId) {
    Object.assign(query, { thirdPartyId: ObjectId(thirdPartyId) });
  }
  return query;
}

function addStoreToThird(storeName, storeId) {
  return {
    $addToSet: {
      stores: {
        storeName,
        storeId: ObjectId(storeId),
      },
    },
  };
}

function findStoreGroupMember(storeId, groupId) {
  return { storeId, groupId };
}

function findStoreGroups(stores) {
  const storeIds = stores.map(item => ObjectId(item));
  return [{
    $match: { storeId: { $in: storeIds } },
  },
  {
    $lookup: {
      from: STORE_GROUPS,
      localField: 'groupId',
      foreignField: '_id',
      as: 'groups',
    },
  },
  {
    $project: {
      groups: 1,
      storeId: 1,
      _id: 0,
    },
  },
  {
    $unwind: { path: '$groups' },
  },
  {
    $project: {
      _id: '$groups._id',
      ownerId: '$groups.ownerId',
      groupName: '$groups.name',
      code: '$groups.code',
      roleRelationId: '$groups.roleRelationId',
      exclusive: '$groups.exclusive',
      enabled: '$groups.enabled',
      creation: '$groups.creation',
      additionalData: '$groups.additionalData',
      storeId: 1,
    },
  },
  ];
}

function findGroup(ownerId, groupName) {
  return { ownerId, groupName };
}

function findOwnerGroups(ownerId) {
  return { ownerId };
}

module.exports = {
  ...commonQueries,
  // Stores
  findStores,
  addStoreToThird,
  findStoreGroupMember,
  findGroup,
  findOwnerGroups,
  findStoreGroups,
};
