const mongodb = require('mongodb');
const common = require('./common');

const { ObjectId } = mongodb;

function findByStore(storeId) {
  return { storeId: ObjectId(storeId) };
}

function findByDevice(deviceId) {
  return { deviceId: ObjectId(deviceId) };
}

function findBySerial(serial) {
  return { serial };
}

function findByThirdParty(thirdPartyId, thirdPartyRole) {
  return {
    thirdParties: {
      $elemMatch: { thirdPartyId: ObjectId(thirdPartyId), thirdPartyRole },
    },
  };
}

function findTerminals(terminalIds) {
  const query = {};
  if (Array.isArray(terminalIds)) {
    const terminalIdsObj = terminalIds.map(item => ObjectId(item));
    Object.assign(query, { _id: { $in: terminalIdsObj } });
  }
  return query;
}

function addExternalRef(objectData) {
  return { $addToSet: { externalCoding: { ...objectData } } };
}

function removeExternalRef(externalSource) {
  return { $pull: { externalCoding: { externalSource } } };
}

function removeThirdParty(thirdPartyRole) {
  return { $pull: { thirdParties: { thirdPartyRole } } };
}

module.exports = {
  ...common,
  ObjectId,
  findTerminals,
  findByStore,
  findBySerial,
  findByDevice,
  findByThirdParty,
  addExternalRef,
  removeExternalRef,
  removeThirdParty,
};
